import unittest
from unittest.mock import patch, Mock

import json
from certbot import errors
from certbot_dns_mijn_host.mijn_host import MijnHostClient


class TestMijnHostClient(unittest.TestCase):
    def setUp(self):
        self.api_key = "test"
        self.client = MijnHostClient(self.api_key)

    @patch("requests.get")
    def test_list_domains(self, mock_get):
        # Mocking the API response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"domains": ["example.com"]}
        mock_get.return_value = mock_response

        # Call the list_domains method
        response = self.client.list_domains()

        # Assert that the GET request was made to the correct URL
        mock_get.assert_called_once_with(
            "https://mijn.host/api/v2/domains", headers={"API-Key": self.api_key}
        )

        # Assert the response is as expected
        self.assertEqual(response, {"domains": ["example.com"]})

    @patch("requests.get")
    def test_get_records(self, mock_get):
        # Mocking the API response
        domain = "example.com"
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "records": [{"type": "TXT", "name": "example.com", "value": "some_value"}]
        }
        mock_get.return_value = mock_response

        # Call the get_records method
        response = self.client.get_records(domain)

        # Assert that the GET request was made to the correct URL
        mock_get.assert_called_once_with(
            f"https://mijn.host/api/v2/domains/{domain}/dns",
            headers={"API-Key": self.api_key},
        )

        # Assert the response is as expected
        self.assertEqual(
            response,
            {
                "records": [
                    {"type": "TXT", "name": "example.com", "value": "some_value"}
                ]
            },
        )

    @patch("requests.get")
    def test_handle_response_non_ok_status(self, mock_get):
        # Mocking the API response with a non-OK status code
        mock_response = Mock()
        mock_response.status_code = 400
        mock_response.text = "Bad Request"
        mock_get.return_value = mock_response

        with self.assertRaises(errors.PluginError) as context:
            self.client.list_domains()

        self.assertIn(
            "Received non-OK status from Mijn Host API", str(context.exception)
        )

    @patch("requests.get")
    def test_handle_response_invalid_json(self, mock_get):
        # Mocking the API response with invalid JSON
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.text = "Invalid JSON"
        mock_response.json.side_effect = json.decoder.JSONDecodeError(
            "Expecting value", "Invalid JSON", 0
        )
        mock_get.return_value = mock_response

        with self.assertRaises(errors.PluginError) as context:
            self.client.list_domains()

        self.assertIn("API response with non-JSON", str(context.exception))


if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages


install_requires = [
    "acme==2.11.0",
    "certbot==2.11.0",
    "certifi==2024.8.30",
    "cffi==1.17.0",
    "charset-normalizer==3.3.2",
    "ConfigArgParse==1.7",
    "configobj==5.0.8",
    "cryptography==43.0.0",
    "distro==1.9.0",
    "idna==3.8",
    "josepy==1.14.0",
    "parsedatetime==2.6",
    "pycparser==2.22",
    "pyOpenSSL==24.2.1",
    "pyRFC3339==1.1",
    "pytz==2024.1",
    "requests==2.32.3",
    "ruff==0.6.3",
    "setuptools==74.0.0",
    "six==1.16.0",
    "urllib3==2.2.2",
]

VERSION = "0.0.2"

DESCRIPTION = "Certbot DNS plugin for mijn.host service, enabling the automation of DNS-01 challenges for issuing wildcard SSL certificates. This plugin simplifies the process of obtaining and renewing SSL certificates by integrating directly with the DNS API of mijn.host Service, making it ideal for system administrators and DevOps professionals managing secure web services."

with open("README.md", "r", encoding="utf-8") as fh:
    LONG_DESCRIPTION = fh.read()

setup(
    name="certbot-dns-mijn-host",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="",
    python_requires=">=2.7",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Plugins",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: Name Service (DNS)",
        "Topic :: Security",
        "Topic :: Security :: Cryptography",
        "Topic :: System :: Networking",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    entry_points={
        "certbot.plugins": [
            "dns-mijn-host = certbot_dns_mijn_host.mijn_host:Authenticator"
        ]
    },
)



from certbot import errors
from certbot.plugins import dns_common
from certbot.plugins.dns_common import CredentialsConfiguration
import requests
import json
import logging
from typing import Any, Optional, Callable
import urllib

logger = logging.getLogger(__name__)

BASE_URL = "https://mijn.host/api/v2/"


class Authenticator(dns_common.DNSAuthenticator):
    def __init__(self, *args, **kwargs):
        super(Authenticator, self).__init__(*args, **kwargs)
        self.credentials: Optional[CredentialsConfiguration] = None
        self.ttl = 15

    description = "This plugin configures a DNS TXT record to respond to a dns-01 challenge using the mijn.host API"

    @classmethod
    def add_parser_arguments(
        cls, add: Callable[..., None], default_propagation_seconds: int = 60
    ) -> None:
        super().add_parser_arguments(add, default_propagation_seconds)
        add("credentials", help="mijn.host credentials INI file.")

    def more_info(self) -> str:
        return "This plugin configures a DNS TXT record to respond to a dns-01 challenge using the mijn.host API."

    def _validate_credentials(self, credentials: CredentialsConfiguration) -> None:
        api_key = credentials.conf("api-key")
        if not api_key:
            raise errors.PluginError("No API key provided")

    def _setup_credentials(self) -> None:
        self.credentials = self._configure_credentials(
            "credentials",
            "mijn-host credentials INI file",
            None,
            self._validate_credentials,
        )

    def _perform(self, domain, validation_name, validation):
        self._get_mijn_host_client().add_txt_record(
            domain, validation_name, validation, self.ttl
        )

    def _cleanup(self, domain, validation_name, validation):
        self._get_mijn_host_client().del_txt_record(domain, validation_name, validation)

    def _get_mijn_host_client(self):
        if not self.credentials:
            raise errors.PluginError("Plugin not initialized")
        return MijnHostClient(self.credentials.conf("api-key"))


class MijnHostClient(object):
    def __init__(self, api_key):
        logger.debug("Creating MijnHostClient")
        self.headers = {
            "API-Key": api_key,
        }

    def _handle_response(self, resp: requests.Response, name: str = "something") -> Any:
        if resp.status_code not in (200, 202):
            print("Failed function name: ", name, resp.text)
            raise errors.PluginError(
                f"Received non-OK status from Mijn Host API {resp.status_code}"
            )
        try:
            return resp.json()
        except json.decoder.JSONDecodeError:
            raise errors.PluginError(
                f"API response with non-json: {resp.text}")

    def get_records(self, domain):
        url = urllib.parse.urljoin(BASE_URL, f"domains/{domain}/dns")
        req = requests.get(url, headers=self.headers)
        resp = self._handle_response(req, name="get records")
        return resp

    def update_records(self, domain, records):
        url = urllib.parse.urljoin(BASE_URL, f"domains/{domain}/dns")
        req = requests.put(
            url, headers=self.headers, data=json.dumps({"records": records})
        )
        resp = self._handle_response(req, name="update records")
        return resp

    def add_txt_record(
        self, domain: str, record_name: str, record_content: str, ttl: int
    ):
        records = self.get_records(domain).get("data", {}).get("records", [])
        new_record = {
            "type": "TXT",
            "name": record_name + ".",
            "value": record_content,
            "ttl": ttl,
        }

        if new_record in records:
            return

        records.append(new_record)
        self.update_records(domain, records)

    def del_txt_record(
        self, domain: str, record_name: str, record_content: str
    ) -> None:
        records = self.get_records(domain).get("data", {}).get("records", [])
        records_list = [
            r
            for r in records
            if not (
                r["type"] == "TXT"
                and r["name"] == record_name + "."
                and r["value"] == record_content
            )
        ]
        self.update_records(domain, records_list)
